@@ -25,9 +25,28 @@ class Nonparametric(object):
     Introduced to complement parametric distributions present in scipy.stats.
     """
 
-    def __init__(self, dist_samples):
+    def __init__(self, dist_samples, correction='clip'):
+        """
+        :Parameters:
+          dist_samples : ndarray
+            Samples to be used to assess the distribution.
+          correction : {'clip'} or None, optional
+            Determines the behavior when .cdf is queried.  If None -- no
+            correction is made.  If 'clip' -- values are clipped to lie
+            in the range [1/(N+2), (N+1)/(N+2)] (simply because
+            non-parametric assessment lacks the power to resolve with
+            higher precision in the tails, so 'imagery' samples are
+            placed in each of the two tails).
+        """
         self._dist_samples = N.ravel(dist_samples)
+        self._correction = correction
 
+    def __repr__(self):
+        return '%s(%r%s)' % (
+            self.__class__.__name__,
+            self._dist_samples,
+            ('', ', correction=%r' % self._correction)
+              [int(self._correction != 'clip')])
 
     @staticmethod
     def fit(dist_samples):
@@ -37,7 +56,18 @@ class Nonparametric(object):
     def cdf(self, x):
         """Returns the cdf value at `x`.
         """
-        return N.vectorize(lambda v:(self._dist_samples <= v).mean())(x)
+        dist_samples = self._dist_samples
+        res = N.vectorize(lambda v:(dist_samples <= v).mean())(x)
+        if self._correction == 'clip':
+            nsamples = len(dist_samples)
+            N.clip(res, 1.0/(nsamples+2), (nsamples+1.0)/(nsamples+2), res)
+        elif self._correction is None:
+            pass
+        else:
+            raise ValueError, \
+                  '%r is incorrect value for correction parameter of %s' \
+                  % (self._correction, self.__class__.__name__)
+        return res
 
 
 def _pvalue(x, cdf_func, tail, return_tails=False, name=None):
@@ -238,6 +268,10 @@ class MCNullDist(NullDist):
         dist_samples = []
         """Holds the values for randomized labels."""
 
+        # Needs to be imported here upon demand due to circular imports
+        # TODO: place MC into a separate module
+        from mvpa.clfs.base import DegenerateInputError, FailedToTrainError
+
         # decide on the arguments to measure
         if not vdata is None:
             measure_args = [vdata, wdata]
@@ -245,10 +279,15 @@ class MCNullDist(NullDist):
             measure_args = [wdata]
 
         # estimate null-distribution
+        skipped = 0                     # # of skipped permutations
         for p in xrange(self.__permutations):
             # new permutation all the time
             # but only permute the training data and keep the testdata constant
             #
+            if __debug__:
+                debug('STATMC', "Doing %i permutations: %i" \
+                      % (self.__permutations, p+1), cr=True)
+
             # TODO this really needs to be more clever! If data samples are
             # shuffled within a class it really makes no difference for the
             # classifier, hence the number of permutations to estimate the
@@ -258,7 +297,19 @@ class MCNullDist(NullDist):
 
             # compute and store the measure of this permutation
             # assume it has `TransferError` interface
-            dist_samples.append(measure(*measure_args))
+            try:
+                dist_samples.append(measure(*measure_args))
+            except (DegenerateInputError, FailedToTrainError), e:
+                if __debug__:
+                    debug('STATMC', " skipped", cr=True)
+                warning("Failed to estimate %s on %s, due to %s. "
+                        "Permutation %d skipped." %
+                        (measure, measure_args, e, p))
+                skipped += 1
+                continue
+
+        if __debug__:
+            debug('STATMC', ' Skipped: %d permutations' % skipped)
 
         # restore original labels
         wdata.permuteLabels(False, perchunk=False)
@@ -495,6 +546,12 @@ if externals.exists('scipy'):
             """Arguments which should get some fixed value"""
 
 
+        def __call__(self, *args, **kwargs):
+            """Upon call mimic call to get actual rv_frozen distribution
+            """
+            return self._dist(*args, **kwargs)
+
+
         def nnlf(self, theta, x):
             # - sum (log pdf(x, theta),axis=0)
             #   where theta are the parameters (including loc and scale)
