@@ -14,6 +14,7 @@ __docformat__ = 'restructuredtext'
 from mvpa.base import externals
 
 import numpy as N
+np = N                      # just for easy merging of changes into >=0.5
 
 from operator import isSequenceType
 
@@ -25,6 +26,15 @@ if externals.exists('scipy', raiseException=True):
     from scipy.linalg import lstsq
     from scipy.special import legendre
 
+    def legendre_(n, x):
+        # Scipy 0.8.0 (and possibly later) has regression of reporting
+        # 'inf's for negative boundary. Lets guard against it for now
+        leg = legendre(n)
+        r = leg(x)
+        infs = np.isinf(r)
+        if np.any(infs):
+            r[infs] = leg(x[infs] + 1e-10) # offset to try to overcome problems
+        return r
 
 @datasetmethod
 def detrend(dataset, perchunk=False, model='linear',
@@ -34,34 +44,35 @@ def detrend(dataset, perchunk=False, model='linear',
     or per each chunk
 
     :Parameters:
-      `dataset` : `Dataset`
+      dataset : Dataset
         dataset to operate on
-      `perchunk` : bool
+      perchunk : bool
         either to operate on whole dataset at once or on each chunk
         separately
-      `model`
+      model
         Type of detrending model to run.  If 'linear' or 'constant',
         scipy.signal.detrend is used to perform a linear or demeaning
         detrend. Polynomial detrending is activated when 'regress' is
         used or when polyord or opt_reg are specified.
-      `polyord` : int or list
+      polyord : int or list
         Order of the Legendre polynomial to remove from the data.  This
         will remove every polynomial up to and including the provided
         value.  For example, 3 will remove 0th, 1st, 2nd, and 3rd order
-        polynomials from the data.  N.B.: The 0th polynomial is the 
+        polynomials from the data.  N.B.: The 0th polynomial is the
         baseline shift, the 1st is the linear trend.
         If you specify a single int and perchunk is True, then this value
-        is used for each chunk.  You can also specify a differnt polyord 
+        is used for each chunk.  You can also specify a different polyord
         value for each chunk by providing a list or ndarray of polyord
         values the length of the number of chunks.
-      `opt_reg` : ndarray
+      opt_reg : ndarray
         Optional ndarray of additional information to regress out from the
         dataset.  One example would be to regress out motion parameters.
         As with the data, time is on the first axis.
 
     """
-    if polyord is not None or opt_reg is not None: model='regress'
-    
+    if polyord is not None or opt_reg is not None:
+        model='regress'
+
     if model in ['linear', 'constant']:
         # perform scipy detrend
         bp = 0                              # no break points by default
@@ -95,24 +106,24 @@ def __detrend_regress(dataset, perchunk=True, polyord=None, opt_reg=None):
     terms as well as optional regressors, such as motion parameters.
 
     :Parameters:
-      `dataset`: `Dataset`
+      dataset : Dataset
         Dataset to operate on
-      `perchunk` : bool
+      perchunk : bool
         Either to operate on whole dataset at once or on each chunk
         separately.  If perchunk is True, all the samples within a
         chunk should be contiguous and the chunks should be sorted in
         order from low to high.
-      `polyord` : int
+      polyord : int
         Order of the Legendre polynomial to remove from the data.  This
         will remove every polynomial up to and including the provided
         value.  For example, 3 will remove 0th, 1st, 2nd, and 3rd order
-        polynomials from the data.  N.B.: The 0th polynomial is the 
+        polynomials from the data.  N.B.: The 0th polynomial is the
         baseline shift, the 1st is the linear trend.
         If you specify a single int and perchunk is True, then this value
-        is used for each chunk.  You can also specify a differnt polyord 
+        is used for each chunk.  You can also specify a different polyord
         value for each chunk by providing a list or ndarray of polyord
         values the length of the number of chunks.
-      `opt_reg` : ndarray
+      opt_reg : ndarray
         Optional ndarray of additional information to regress out from the
         dataset.  One example would be to regress out motion parameters.
         As with the data, time is on the first axis.
@@ -139,24 +150,24 @@ def __detrend_regress(dataset, perchunk=True, polyord=None, opt_reg=None):
             # get the indices for that chunk
             cinds = dataset.chunks == chunk
 
-            # see if add in polyord values    
+            # see if add in polyord values
             if not polyord is None:
                 # create the timespan
                 x = N.linspace(-1, 1, cinds.sum())
                 # create each polyord with the value for that chunk
                 for n in range(polyord[n] + 1):
                     newreg = N.zeros((dataset.nsamples, 1))
-                    newreg[cinds, 0] = legendre(n)(x)
+                    newreg[cinds, 0] = legendre_(n, x)
                     reg.append(newreg)
     else:
         # take out mean over entire dataset
         reg = []
-        # see if add in polyord values    
+        # see if add in polyord values
         if not polyord is None:
             # create the timespan
             x = N.linspace(-1, 1, dataset.nsamples)
             for n in range(polyord[0] + 1):
-                reg.append(legendre(n)(x)[:, N.newaxis])
+                reg.append(legendre_(n, x)[:, N.newaxis])
 
     # see if add in optional regs
     if not opt_reg is None:
